@@ -8,10 +8,18 @@
  * This is free software, see COPYING fro details
  *
  */
+#if ENABLE_NUNIT
+using NUnit.Framework;
+#endif
+
 namespace FSpot.Filters {
 	public class OrientationFilter : IFilter {
-		public bool Convert (string source, string dest)
+		public bool Convert (FilterRequest req)
 		{
+			string source = req.Current.LocalPath;
+			System.Uri dest_uri = req.TempUri (System.IO.Path.GetExtension (source));
+			string dest = dest_uri.LocalPath;
+
 			ImageFile img = ImageFile.Create (source);
 			bool changed = false;
 			
@@ -51,7 +59,24 @@ namespace FSpot.Filters {
 				jimg.SaveMetaData (dest);
 			}
 
+			if (changed)
+				req.Current = dest_uri;
+
 			return changed;
 		}
+		
+#if ENABLE_NUNIT
+		[TestFixture]
+		public class Tests : ImageTest {
+			[Test]
+			public void TestNoop ()
+			{
+				string path = CreateFile ("test.jpg", 50);
+				FilterRequest req = new FilterRequest (path);
+				IFilter filter = new OrientationFilter ();
+				Assert.IsFalse (filter.Convert (req), "Orientation Filter changed a normal file");
+			}
+		}
+#endif
 	}
 }
