@@ -2,7 +2,8 @@ using FlickrNet;
 using System;
 using System.IO;
 using System.Threading;
-using Mono.Posix;
+using Mono.Unix;
+using FSpot.Filters;
 
 namespace FSpot {
 	public class FlickrExport : GladeDialog {
@@ -41,6 +42,7 @@ namespace FSpot {
 		Auth auth;
 
 		FlickrRemote fr;
+		private FlickrRemote.Service current_service;
 
 		string auth_text;
 		private State state;
@@ -65,33 +67,40 @@ namespace FSpot {
 				case State.Connected:
 					auth_flickr.Sensitive = true;
 					do_export_flickr.Sensitive = false;
-					auth_label.Text = Catalog.GetString ("Return to this window after you have finished the authorization process on Flickr.com and click the \"Complete Authorization\" button below");
+					auth_label.Text = string.Format (Catalog.GetString ("Return to this window after you have finished the authorization process on {0} and click the \"Complete Authorization\" button below"), current_service.Name);
 					auth_flickr.Label = Catalog.GetString ("Complete Authorization");
 					break;
 				case State.InAuth:
 					auth_flickr.Sensitive = false;
-					auth_label.Text = Catalog.GetString ("Logging into Flickr.Com");
+					auth_label.Text = string.Format (Catalog.GetString ("Logging into {0}"), current_service.Name);
 					auth_flickr.Label = Catalog.GetString ("Checking credentials...");
 					do_export_flickr.Sensitive = false;
 					break;
 				case State.Authorized:
 					do_export_flickr.Sensitive = true;
 					auth_flickr.Sensitive = true;
-					auth_label.Text = System.String.Format (Catalog.GetString ("Welcome {0} you are connected to Flickr.Com"),
-										auth.User.UserName);
-					auth_flickr.Label = String.Format (Catalog.GetString ("Sign in as a different user"), auth.User.UserName);
+					auth_label.Text = System.String.Format (Catalog.GetString ("Welcome {0} you are connected to {1}"),
+										auth.User.Username,
+										current_service.Name);
+					auth_flickr.Label = String.Format (Catalog.GetString ("Sign in as a different user"), auth.User.Username);
 					break;
 				}
 				state = value;
 			}
 		}
 
+		public FlickrExport (IBrowsableCollection selection, bool display_tags) :
+			this (SupportedService.Flickr, selection, display_tags)
+		{ }
 
-		public FlickrExport (IBrowsableCollection selection) : base ("flickr_export_dialog")
+
+		public FlickrExport (SupportedService service, IBrowsableCollection selection, bool display_tags) : base ("flickr_export_dialog")
 		{
 			this.selection = selection;
+			this.current_service = FlickrRemote.Service.FromSupported (service);
 
 			IconView view = new IconView (selection);
+			view.DisplayTags = display_tags;
 			view.DisplayDates = false;
 
 			Dialog.Modal = false;
@@ -105,20 +114,21 @@ namespace FSpot {
 			Dialog.ShowAll ();
 			Dialog.Response += HandleResponse;
 			auth_flickr.Clicked += HandleClicked;
-			auth_text = auth_label.Text;
+			auth_text = string.Format (auth_label.Text, current_service.Name);
+			auth_label.Text = auth_text;
 
 			LoadPreference (Preferences.EXPORT_FLICKR_SCALE);
 			LoadPreference (Preferences.EXPORT_FLICKR_SIZE);
 			LoadPreference (Preferences.EXPORT_FLICKR_BROWSER);
 			LoadPreference (Preferences.EXPORT_FLICKR_TAGS);
 			LoadPreference (Preferences.EXPORT_FLICKR_STRIP_META);
-			LoadPreference (Preferences.EXPORT_FLICKR_TOKEN);
 			LoadPreference (Preferences.EXPORT_FLICKR_PUBLIC);
 			LoadPreference (Preferences.EXPORT_FLICKR_FAMILY);
 			LoadPreference (Preferences.EXPORT_FLICKR_FRIENDS);
+			LoadPreference (current_service.PreferencePath);
 
 			do_export_flickr.Sensitive = false;
-			fr = new FlickrRemote (token);			
+			fr = new FlickrRemote (token, current_service);
 			if (token != null && token.Length > 0) {
 				StartAuth ();
 			}
@@ -143,16 +153,16 @@ namespace FSpot {
 			} catch (FlickrException e) {
 				args.Exception = e;
 			}
-			
+
 			Gtk.Application.Invoke (this, args, delegate (object sender, EventArgs sargs) {
-				AuthorizationEventArgs args = (AuthorizationEventArgs) sargs;
-				
-				do_export_flickr.Sensitive = args.Auth != null;
-				if (args.Auth != null) {
-					token = args.Auth.Token;
-					auth = args.Auth;
+				AuthorizationEventArgs wargs = (AuthorizationEventArgs) sargs;
+
+				do_export_flickr.Sensitive = wargs.Auth != null;
+				if (wargs.Auth != null) {
+					token = wargs.Auth.Token;
+					auth = wargs.Auth;
 					CurrentState = State.Authorized;
-					Preferences.Set (Preferences.EXPORT_FLICKR_TOKEN, fr.Token);
+					Preferences.Set (current_service.PreferencePath, token);
 				} else {
 					CurrentState = State.Disconnected;
 				}
@@ -187,27 +197,33 @@ namespace FSpot {
 		{
 			token = null;
 			auth = null;
-			fr = new FlickrRemote (token);
-			Preferences.Set (Preferences.EXPORT_FLICKR_TOKEN, "");
+			fr = new FlickrRemote (token, current_service);
+			Preferences.Set (current_service.PreferencePath, "");
 			CurrentState = State.Disconnected;
 		}
 
 		private void Login () 
 		{
 			try {
-				fr = new FlickrRemote (token);
+				fr = new FlickrRemote (token, current_service);
 				fr.TryWebLogin();
 				CurrentState = State.Connected;
 			} catch (FlickrException e) {
-				HigMessageDialog md = 
-					new HigMessageDialog (Dialog, 
-							      Gtk.DialogFlags.Modal |
-							      Gtk.DialogFlags.DestroyWithParent,
-							      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
-							      Catalog.GetString ("Unable to log on"), e.Message);
-				md.Run ();
-				md.Destroy ();
-				CurrentState = State.Disconnected;
+				if (e.Code == 98) {
+					Logout ();
+					Login ();
+				} else {
+					HigMessageDialog md = 
+						new HigMessageDialog (Dialog, 
+								      Gtk.DialogFlags.Modal |
+								      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+								      Catalog.GetString ("Unable to log on"), e.Message);
+
+					md.Run ();
+					md.Destroy ();
+					CurrentState = State.Disconnected;
+				}
 			}
 		}
 
@@ -246,22 +262,33 @@ namespace FSpot {
 						selection.Count);
 					
 					info = new FileInfo (photo.DefaultVersionUri.LocalPath);
-					string id = fr.Upload (photo, scale, size, copy_metadata, is_public, is_family, is_friend);
+					FilterSet stack = new Filters.FilterSet ();
+					if (scale)
+						stack.Add (new ResizeFilter ((uint)size));
+					
+					string id = fr.Upload (photo, stack, is_public, is_family, is_friend);
 					ids.Add (id);
+
+                                        Core.Database.Exports.Create ((photo as Photo).Id,
+								      (photo as Photo).DefaultVersionId,
+								      ExportStore.FlickrExportType,
+								      auth.User.UserId+":"+auth.User.Username + ":" + id);
+                                        
 					progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
 					progress_dialog.Fraction = 1.0;
 					progress_dialog.ProgressText = Catalog.GetString ("Upload Complete");
 					progress_dialog.ButtonLabel = Gtk.Stock.Ok;
 				}
 			} catch (System.Exception e) {
-				progress_dialog.Message = String.Format (Catalog.GetString ("Error Uploading To Flickr: {0}"),
+				progress_dialog.Message = String.Format (Catalog.GetString ("Error Uploading To {0}: {1}"),
+									 current_service.Name,
 									 e.Message);
 				progress_dialog.ProgressText = Catalog.GetString ("Error");
 				System.Console.WriteLine (e);
 			}
 
 			if (open && ids.Count != 0) {
-				string view_url = "http://www.flickr.com/tools/uploader_edit.gne?ids";
+				string view_url = string.Format ("http://www.{0}/tools/uploader_edit.gne?ids", current_service.Name);
 				bool first = true;
 
 				foreach (string id in ids) {
@@ -316,7 +343,8 @@ namespace FSpot {
 							      Gtk.DialogFlags.DestroyWithParent,
 							      Gtk.MessageType.Error, Gtk.ButtonsType.Ok, 
 							      Catalog.GetString ("Unable to log on."),
-							      Catalog.GetString ("F-Spot was unable to log on to Flickr.  Make sure you have given the authentication using Flickr web browser interface."));
+							      string.Format (Catalog.GetString ("F-Spot was unable to log on to {0}.  Make sure you have given the authentication using {0} web browser interface."),
+							      		     current_service.Name));
 				md.Run ();
 				md.Destroy ();
 				return;
@@ -325,7 +353,7 @@ namespace FSpot {
 			fr.ExportTags = tag_check.Active;
 			open = open_check.Active;
 			scale = scale_check.Active;
-			copy_metadata = open_check.Active;
+			copy_metadata = !meta_check.Active;
 			is_public = public_radio.Active;
 			is_family = family_check.Active;
 			is_friend = friend_check.Active;
@@ -345,10 +373,10 @@ namespace FSpot {
 			Preferences.Set (Preferences.EXPORT_FLICKR_BROWSER, open);
 			Preferences.Set (Preferences.EXPORT_FLICKR_TAGS, tag_check.Active);
 			Preferences.Set (Preferences.EXPORT_FLICKR_STRIP_META, meta_check.Active);
-			Preferences.Set (Preferences.EXPORT_FLICKR_TOKEN, fr.Token);
 			Preferences.Set (Preferences.EXPORT_FLICKR_PUBLIC, public_radio.Active);
 			Preferences.Set (Preferences.EXPORT_FLICKR_FAMILY, family_check.Active);
 			Preferences.Set (Preferences.EXPORT_FLICKR_FRIENDS, friend_check.Active);
+			Preferences.Set (current_service.PreferencePath, fr.Token);
 		}
 
 		void LoadPreference (string key)
@@ -381,7 +409,9 @@ namespace FSpot {
 				if (meta_check.Active != (bool) val)
 					meta_check.Active = (bool) val;
 				break;
-			case Preferences.EXPORT_FLICKR_TOKEN:
+			case Preferences.EXPORT_TOKEN_FLICKR:
+			case Preferences.EXPORT_TOKEN_23HQ:
+			case Preferences.EXPORT_TOKEN_ZOOOMR:
 				token = (string) val;
 			        break;
 			case Preferences.EXPORT_FLICKR_PUBLIC:
@@ -409,6 +439,6 @@ namespace FSpot {
 				*/
 			}
 		}
-	}
 
+	}
 }
