@@ -6,79 +6,23 @@ using Cairo;
 using Mono.Unix;
 
 namespace FSpot {
-	public class FadeIn {
-		bool composited;
-		Delay fade_delay;
-		TimeSpan duration;
-		DateTime start;
-		Gtk.Window win;
-		
-		public FadeIn (Gtk.Window win, int sec)
-		{
-			this.win = win;
-			win.Mapped += HandleMapped;
-			win.Unmapped += HandleUnmapped;
-			win.ExposeEvent += HandleExposeEvent;
-			duration = new TimeSpan (0, 0, sec);
-		}
-		
-		[GLib.ConnectBefore]
-		public void HandleMapped (object sender, EventArgs args)
-		{
-			composited = CompositeUtils.SupportsHint (win.Screen, "_NET_WM_WINDOW_OPACITY");
-			if (!composited)
-				return;
-			
-			CompositeUtils.SetWinOpacity (win, 0.0);
-		}
-		
-		public void HandleExposeEvent (object sender, ExposeEventArgs args)
-		{
-			if (fade_delay == null) {
-				fade_delay = new Delay (50, new GLib.IdleHandler (Update));
-					start = DateTime.Now;
-					fade_delay.Start ();
-			}
-		}
-		
-		public void HandleUnmapped (object sender, EventArgs args)
-		{
-			if (fade_delay != null)
-				fade_delay.Stop ();
-		}
-		
-		public bool Update ()
-		{
-			double percent = Math.Min ((DateTime.Now - start).Ticks / (double) duration.Ticks, 1.0);
-			double opacity = Math.Sin (percent * Math.PI * 0.5);
-			CompositeUtils.SetWinOpacity (win, opacity);
-			
-			bool stop = opacity >= 1.0;
-
-			if (stop)
-				fade_delay.Stop ();
-			
-			return !stop;
-		}
-	}			
-	
 	[Binding(Gdk.Key.Escape, "Quit")]
-#if ENABLE_CRACK
-	[Binding(Gdk.Key.D, "PlayPause")]
-#endif
 	public class FullScreenView : Gtk.Window {
 		private ScrolledView scroll;
 		private PhotoImageView view;
 		private TagView tag_view;
+		private Label name_label;
 		private Notebook notebook;
-#if ENABLE_CRACK
-		private ImageDisplay display;
-#endif		
+		private ControlOverlay controls;
+		//		private ImageDisplay display;
+		private Widget display;
+
 		ActionGroup actions;
 		const string ExitFullScreen = "ExitFullScreen";
-		const string NextPicture = "NextPicture";
-		const string PreviousPicture = "PreviousPicture";
-
+		const string HideToolbar = "HideToolbar";
+		const string SlideShow = "SlideShow";
+		const string Info = "Info";
+		
 		public FullScreenView (IBrowsableCollection collection) : base ("Full Screen Mode")
 		{
 			try {
@@ -86,16 +30,41 @@ namespace FSpot {
 				actions = new ActionGroup ("joe");
 				
 				actions.Add (new ActionEntry [] {
-					new ActionEntry (ExitFullScreen, Stock.Quit, Catalog.GetString ("Exit fullscreen"), null, null, new System.EventHandler (ExitAction)),
-					new ActionEntry (NextPicture, Stock.GoForward, Catalog.GetString ("Next"), null, Catalog.GetString ("Next Picture"), new System.EventHandler (NextAction)),
-					new ActionEntry (PreviousPicture, Stock.GoBack, Catalog.GetString ("Back"), null, Catalog.GetString ("Previous Picture"), new System.EventHandler (PreviousAction))
+					new ActionEntry (HideToolbar, Stock.Close, 
+							 Catalog.GetString ("Hide"), 
+							 null, 
+							 Catalog.GetString ("Hide Toolbar"), 
+							 HideToolbarAction),
+					new ActionEntry (ExitFullScreen, 
+							 Stock.Quit, 
+							 Catalog.GetString ("Exit fullscreen"), 
+							 null, 
+							 null, 
+							 ExitAction),
+						new ActionEntry (SlideShow,
+								 "f-spot-slideshow",
+								 Catalog.GetString ("Slideshow"),
+								 null,
+								 Catalog.GetString ("Start slideshow"),
+								 SlideShowAction),
 				});
 
-				new FadeIn (this, 3);
+				actions.Add (new ToggleActionEntry [] {
+					new ToggleActionEntry (Info,
+							       Stock.Info,
+							       Catalog.GetString ("Info"),
+							       null,
+							       Catalog.GetString ("Image Information"),
+							       InfoAction,
+							       false)
+						});
+				
+				new Fader (this, 1.0, 3);
 				notebook = new Notebook ();
 				notebook.ShowBorder = false;
 				notebook.ShowTabs = false;
 				notebook.Show ();
+				
 
 				scroll = new ScrolledView ();
 				view = new PhotoImageView (collection);
@@ -105,22 +74,38 @@ namespace FSpot {
 				this.Add (notebook);
 				view.Show ();
 				view.MotionNotifyEvent += HandleViewMotion;
+				
+				Action rotate_left = new RotateLeftAction (view.Item);
+				actions.Add (rotate_left);
+				
+				Action rotate_right = new RotateRightAction (view.Item);
+				actions.Add (rotate_right);
 
 				scroll.ScrolledWindow.Add (view);
 				HBox hhbox = new HBox ();
-				Gtk.Button close = ExitButton ();
-				hhbox.PackStart (close);
-				hhbox.PackStart (GetButton ("PreviousPicture"));
-				hhbox.PackStart (GetButton ("NextPicture"));
-
+				hhbox.PackEnd (GetButton (HideToolbar), false, true, 0);
+				hhbox.PackEnd (GetButton (Info), false, true, 0);
+				hhbox.PackEnd (GetButton (SlideShow), false, true, 0);
+				hhbox.PackStart (GetButton (ExitFullScreen), false, false, 0);
+				hhbox.PackStart (Add (new PreviousPictureAction (view.Item)), false, false, 0);
+				hhbox.PackStart (Add (new NextPictureAction (view.Item)), false, false, 0);
+				hhbox.PackStart (Add (new RotateLeftAction (view.Item)), false, false, 0);
+				hhbox.PackStart (Add (new RotateRightAction (view.Item)), false, false, 0);
+				hhbox.BorderWidth = 15;
+				
+				VBox vbox = new VBox ();
+				name_label = new Label ();
+				name_label.UseMarkup = true;
+				vbox.PackStart (name_label);
+				Alignment center = new Alignment (.5f, .5f, 1f, 1f);
 				tag_view = new TagView ();
-				tag_view.Show ();
-				hhbox.PackStart (tag_view);
-				//hhbox.PackStart (new Gtk.Label ("This is a test"));
-				scroll.ControlBox.Add (hhbox);
+				center.Add (tag_view);
+				vbox.PackStart (center);
+				vbox.Spacing = 10;
+
+				hhbox.PackStart (vbox, true, true, 5);
+
 				hhbox.ShowAll ();
-				close.Clicked += ExitAction;
-				close.Show ();
 				//scroll.ShowControls ();
 				
 				scroll.Show ();
@@ -130,16 +115,28 @@ namespace FSpot {
 				
 				view.Item.Changed += HandleItemChanged;
 				view.GrabFocus ();
+				
+				controls = new ControlOverlay (this);
+				controls.Add (hhbox);
+				controls.Dismiss ();
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e);
-			}		      
+			}	
+
+		}
+
+		private Widget Add (Action action)
+		{
+			actions.Add (action);
+			return GetButton (action.Name);
 		}
 
 		private void HandleItemChanged (object sender, BrowsablePointerChangedArgs args)
 		{
 			tag_view.Current = view.Item.Current;
-			actions [NextPicture].Sensitive = view.Item.Index < view.Item.Collection.Count -1;
-			actions [PreviousPicture].Sensitive = view.Item.Index > 0;
+			name_label.Markup = String.Format ("<small>{0}</small>",
+							   view.Item.Current != null ? view.Item.Current.Name : "");
+
 			if (scroll.ControlBox.Visible)
 				scroll.ShowControls ();
 		}
@@ -170,7 +167,15 @@ namespace FSpot {
 		{
 			Action action = actions [name];
 			Widget w = action.CreateIcon (IconSize.Button);
-			Button button = new Button ();
+			Button button;
+			if (action is ToggleAction) {
+				ToggleButton toggle = new ToggleButton ();
+				toggle.Active = ((ToggleAction)action).Active;
+				button = toggle;
+			} else {
+				button = new Button ();
+			}
+			button.Relief = ReliefStyle.None;
 			button.Add (w);
 			w.Show ();
 
@@ -183,14 +188,30 @@ namespace FSpot {
 			this.Destroy ();
 		}
 
-		private void NextAction (object sender, System.EventArgs args)
+	        private void HideToolbarAction (object sender, System.EventArgs args)
 		{
-			view.Item.MoveNext ();
+			scroll.HideControls (true);
+			controls.Dismiss ();
 		}
 
-		private void PreviousAction (object sender, System.EventArgs args)
+		private void SlideShowAction (object sender, System.EventArgs args)
 		{
-			view.Item.MovePrevious ();
+			PlayPause ();
+		}
+
+		InfoOverlay info;
+		private void InfoAction (object sender, System.EventArgs args)
+		{
+			ToggleAction action = sender as ToggleAction;
+
+			if (info == null) {
+				info = new InfoOverlay (this, view.Item);
+			}
+
+			System.Console.WriteLine ("sender = {0}", sender);
+			info.Visibility = action.Active ? 
+				ControlOverlay.VisibilityType.Partial : 
+				ControlOverlay.VisibilityType.None;
 		}
 
 		[GLib.ConnectBefore]
@@ -198,7 +219,8 @@ namespace FSpot {
 		{
 			int x, y;
 			Gdk.ModifierType type;
-			view.GdkWindow.GetPointer (out x, out y, out type);
+			((Gtk.Widget)sender).GdkWindow.GetPointer (out x, out y, out type);
+			controls.Visibility = ControlOverlay.VisibilityType.Partial;
 			scroll.ShowControls ();
 		}
 
@@ -217,21 +239,24 @@ namespace FSpot {
 			}
 		}
 
-#if ENABLE_CRACK
 		public bool PlayPause ()
 		{
 			if (display == null) {
-				display = new ImageDisplay (view.Item);
+				//display = new ImageDisplay (view.Item);
+				display = new TextureDisplay (view.Item);
+				display.AddEvents ((int) (Gdk.EventMask.PointerMotionMask));
+				display.MotionNotifyEvent += HandleViewMotion;
 				notebook.AppendPage (display, null);
 				display.Show ();
 			}
+
 			if (notebook.CurrentPage == 0)
 				notebook.CurrentPage = 1;
 			else
 				notebook.CurrentPage = 0;
 			return true;
 		}
-#endif
+
 		public void Quit ()
 		{
 			this.Destroy ();
@@ -239,6 +264,8 @@ namespace FSpot {
 
 		protected override bool OnKeyPressEvent (Gdk.EventKey key)
 		{
+			controls.Visibility = ControlOverlay.VisibilityType.Partial;
+
 			if (key == null) {
 				System.Console.WriteLine ("Key == null", key);
 				return false;
