@@ -319,11 +319,13 @@ class PixbufUtils {
 			terminated_values = new string [values.Length + 1];
 			Array.Copy (values, terminated_values, values.Length);
 		}
-		
+
+#if false		
 		if (terminated_options != null)
 			System.Console.WriteLine ("options = {0}, values = {1}", terminated_options [0], terminated_values [0]);
 		else
 			Console.WriteLine ("no options");
+#else
 
 		bool success = gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
 							     out data, 
@@ -343,6 +345,7 @@ class PixbufUtils {
 		Marshal.Copy (data, content, 0, (int)length);
 
 		return content;
+#endif
 	}
 	
 	public static Pixbuf TagIconFromPixbuf (Pixbuf source)
@@ -574,17 +577,62 @@ class PixbufUtils {
 		return ret;
 	}	
 	
+	[DllImport ("libfspot")]
+	static extern IntPtr f_pixbuf_blur (IntPtr src, double radius);
+
+	public static Pixbuf Blur (Pixbuf src, double radius)
+	{
+		IntPtr raw_ret = f_pixbuf_blur (src.Handle, radius);
+ 		Gdk.Pixbuf ret = (Gdk.Pixbuf) GLib.Object.GetObject(raw_ret, true);
+		return ret;
+	}	
+
+#if OLDREDEYE
 	[DllImport ("libfspot")]
 	static extern void f_pixbuf_remove_redeye (IntPtr src);
-	
+
 	public static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area)
+#else
+	public unsafe static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area)
+	{
+		return RemoveRedeye (src, area, -15);
+	}
+
+	public unsafe static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area, int threshold)
+	//threshold, factors and comparisons borrowed from the gimp plugin 'redeye.c' by Robert Merkel
+#endif
 	{
 		Gdk.Pixbuf copy = src.Copy ();
 		Gdk.Pixbuf selection = new Gdk.Pixbuf (copy, area.X, area.Y, area.Width, area.Height);
-
+#if OLREDEYE
 		f_pixbuf_remove_redeye (selection.Handle);
 		selection.Dispose ();
+#else
+		byte *spix = (byte *)selection.Pixels;
+		int h = selection.Height;
+		int w = selection.Width;
+		int channels = src.NChannels;
 
+		double RED_FACTOR = 0.5133333;
+		double GREEN_FACTOR = 1;
+		double BLUE_FACTOR = 0.1933333;
+
+		for (int j = 0; j < h; j++) {
+			byte *s = spix;
+			for (int i = 0; i < w; i++) {
+				int adjusted_red = (int)(s[0] * RED_FACTOR);
+				int adjusted_green = (int)(s[1] * GREEN_FACTOR);
+				int adjusted_blue = (int)(s[2] * BLUE_FACTOR);
+
+				if (adjusted_red >= adjusted_green - threshold
+				    && adjusted_red >= adjusted_blue - threshold)
+					s[0] = (byte)(((double)(adjusted_green + adjusted_blue)) / (2.0 * RED_FACTOR));
+				s += channels;
+			}
+			spix += selection.Rowstride;
+		}
+
+#endif
 		return copy;
 	}
 
@@ -646,6 +694,56 @@ class PixbufUtils {
 		
 	}
 
+	public static unsafe bool IsGray (Gdk.Pixbuf pixbuf, int max_difference)
+	{
+		int chan = pixbuf.NChannels;
+
+		byte *pix = (byte *)pixbuf.Pixels;
+		int h = pixbuf.Height;
+		int w = pixbuf.Width;
+		int stride = pixbuf.Rowstride;
+
+		for (int j = 0; j < h; j++) {
+			byte *p = pix;
+			for (int i = 0; i < w; i++) {
+				if (Math.Abs (p[0] - p[1]) > max_difference || Math.Abs (p[0] - p [2]) > max_difference) {
+					goto Found;
+				}
+				p += chan;
+			}
+			pix += stride;
+		}
+
+		return true;
+
+	Found:
+		return false;
+	}
+
+	public static unsafe void ReplaceColor (Gdk.Pixbuf src, Gdk.Pixbuf dest)
+	{
+		if (src.HasAlpha || !dest.HasAlpha || (src.Width != dest.Width) || (src.Height != dest.Height))
+			throw new ApplicationException ("invalid pixbufs");
+
+		byte *dpix = (byte *)dest.Pixels;
+		byte *spix = (byte *)src.Pixels;
+		int h = src.Height;
+		int w = src.Width;
+		for (int j = 0; j < h; j++) {
+			byte *d = dpix;
+			byte *s = spix;
+			for (int i = 0; i < w; i++) {
+				d[0] = s[0];
+				d[1] = s[1];
+				d[2] = s[2];
+				d += 4;
+				s += 3;
+			}
+			dpix += dest.Rowstride;
+			spix += src.Rowstride;
+		}
+	}
+
 	public static Gdk.Pixbuf GetThumbnail (Exif.ExifData data)
 	{
 		byte [] thumb_data = data.Data;
