@@ -74,10 +74,10 @@ namespace FSpot.Widgets {
 		[DllImport("libfspot")]
 		static extern IntPtr f_pixbuf_to_cairo_surface (IntPtr handle);
 
-		public static Surface CreateSurface (Gdk.Pixbuf pixbuf)
+		public static MemorySurface CreateSurface (Gdk.Pixbuf pixbuf)
 		{
 			IntPtr surface = f_pixbuf_to_cairo_surface (pixbuf.Handle);
-			return Surface.LookupExternalSurface (surface);
+			return new MemorySurface (surface);
 		}
 
 		public static Surface CreateSurface (Gdk.Drawable d)
@@ -90,5 +90,44 @@ namespace FSpot.Widgets {
 							       width, height);
 			return surface;
 		}
+		
+		public static Surface CreateGlitzSurface (Gdk.Drawable d)
+		{
+
+			Console.WriteLine ("XvisID: " + GdkUtils.GetXVisualId (d.Visual));
+			IntPtr fmt = NDesk.Glitz.GlitzAPI.glitz_glx_find_drawable_format_for_visual (GdkUtils.GetXDisplay (d.Display), 
+												     d.Screen.Number, 
+												     GdkUtils.GetXVisualId (d.Visual));
+			
+			Console.WriteLine ("fmt: " + fmt);
+			
+			uint w = 100, h = 100;
+			IntPtr glitz_drawable = NDesk.Glitz.GlitzAPI.glitz_glx_create_drawable_for_window (GdkUtils.GetXDisplay (d.Display),
+													   d.Screen.Number, 
+													   fmt,      
+													   GdkUtils.GetXid (d), w, h);
+			
+			NDesk.Glitz.Drawable ggd = new NDesk.Glitz.Drawable (glitz_drawable);
+			IntPtr glitz_format = ggd.FindStandardFormat (NDesk.Glitz.FormatName.ARGB32);
+			
+			NDesk.Glitz.Surface ggs = new NDesk.Glitz.Surface (ggd, glitz_format, 100, 100, 0, IntPtr.Zero);
+			Console.WriteLine (ggd.Features);
+			bool doublebuffer = false;
+			ggs.Attach (ggd, doublebuffer ? NDesk.Glitz.DrawableBuffer.BackColor : NDesk.Glitz.DrawableBuffer.FrontColor);
+			
+			//GlitzAPI.glitz_drawable_destroy (glitz_drawable);
+			GlitzSurface gs = new GlitzSurface (ggs.Handle);
+			
+			return gs;
+		}
+
+		[DllImport("libfspot")]
+		static extern IntPtr f_pixbuf_from_cairo_surface (IntPtr handle);
+		
+		public static Gdk.Pixbuf CreatePixbuf (Surface s)
+		{
+			IntPtr result = f_pixbuf_from_cairo_surface (s.Handle);
+			return (Gdk.Pixbuf) GLib.Object.GetObject (result, true);
+		}
 	}
 }
